----------------------------------------------------------------------------------
-- Company: DESY
-- Engineer: F. Costanza
-- 
-- Create Date:    16:19:16 01/25/2016 
-- Design Name:    debug pins mux
-- Module Name:    debug_mux - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: Thsi is a module that allows to select a signal among several inputs.
-- The main module is "debug_mux" which uses "global_mux" to select a signal from an SFP
-- or a global signal through "global_signal_sel".
-- If an SFP signal is requested (when global_signal_sel = 1), "sfp_sel" is used to select
-- the SFP. The actual signal is selected usig "sfp_signal_mux" controlled by "sfp_signal_sel"
--
-- Dependencies: 
--
-- Revision: 1.0
-- Additional Comments: 
--
----------------------------------------------------------------------------------

-- select sfp signal
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use ieee.numeric_std.all;

entity sfp_signal_mux is
port(
	sel : in std_logic_vector(5 downto 0);
	in1: in std_logic;
	in2 : in std_logic;
	in3 : in std_logic;
   in4 : in std_logic;
	in5 : in std_logic;
   in6 : in std_logic;
	in7 : in std_logic;
	in8 : in std_logic;
	in9 : in std_logic;
	in10 : in std_logic;
	o : out std_logic	
);
end sfp_signal_mux;

architecture Behavioral of sfp_signal_mux is
begin
	o <= in1 when sel = "000001" else
		  in2 when sel = "000010" else
		  in3 when sel = "000011" else
		  in4 when sel = "000100" else
		  in5 when sel = "000101" else
		  in6 when sel = "000110" else
		  in7 when sel = "000111" else
		  in8 when sel = "001000" else
		  in9 when sel = "001001" else
		  in10 when sel = "001010" else
		  '0'; 
end Behavioral;

-- select global signal
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use ieee.numeric_std.all;

entity global_mux is
port(
	sel : in std_logic_vector(6 downto 0);
	in1 : in std_logic;
	in2 : in std_logic;
	in3 : in std_logic;
	in4 : in std_logic;
	o : out std_logic	
);
end global_mux;

architecture Behavioral of global_mux is
begin
	o <= in1 when sel = "0000001" else
		  in2 when sel = "0000010" else
		  in3 when sel = "0000011" else
		  in4 when sel = "0000100" else
		  '0'; 
end Behavioral;


-- component to be called from ngFEC
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use ieee.numeric_std.all;
USE work.gbt_bank_package.ALL;

entity debug_mux is
port(
	global_signal_sel : in std_logic_vector(6 downto 0);
	sfp_signal_sel : in std_logic_vector(5 downto 0);
	sfp_sel : in std_logic_vector(2 downto 0);
	
	ttc_clk : in std_logic;
	qie_reset : in std_logic;
	tx_wordclk : in std_logic;
	tx_frameclk : in std_logic_vector(0 to 7);
	rx_frameclk : in std_logic_vector(0 to 7);	
	rx_wordclk : in std_logic_vector(0 to 7);
	rx_frameclk_locked : in std_logic_vector(0 to 7);	
	rx_wordclk_locked : in std_logic_vector(0 to 7);
	--rx_header_flag : in std_logic_vector(0 to 7);
    --gbt_tx : in data_nx84bit_a(0 to 7);
	--gbt_rx : in data_nx84bit_a(0 to 7);
	sfp_sda : in std_logic_vector(0 to 7);
	sfp_scl : in std_logic_vector(0 to 7);
	
	debug_out : out std_logic	
);
end debug_mux;

architecture Behavioral of debug_mux is

component sfp_signal_mux is
	port(
		sel : in std_logic_vector(5 downto 0);
		in1: in std_logic;
		in2 : in std_logic;
		in3 : in std_logic;
		in4 : in std_logic;
		in5 : in std_logic;
		in6 : in std_logic;
		in7 : in std_logic;
		in8 : in std_logic;
		in9 : in std_logic;
		in10 : in std_logic;
		
		o : out std_logic	
	);
end component sfp_signal_mux;

component global_mux is
	port(
		sel : in std_logic_vector(6 downto 0);
		in1: in std_logic;
		in2 : in std_logic;
		in3 : in std_logic;
		in4 : in std_logic;
		
		o : out std_logic	
	);
end component global_mux;

signal sfp_sel_int : integer range 0 TO 7;
signal sfp_signals : std_logic_vector(0 to 7);
signal sfp_out : std_logic;

begin
	 debug_sfp_signal_mux_gen: FOR sfp IN 0 TO 7 GENERATE 		
	   debug_sfp_signal_mux : sfp_signal_mux
      PORT MAP (
			sel  => sfp_signal_sel,
			in1  => tx_frameclk(sfp),
			in2  => rx_frameclk(sfp),
			in3  => rx_wordclk(sfp),
			in4  => rx_frameclk_locked(sfp),
			in5  => rx_wordclk_locked(sfp),
			in6  => '0',--rx_header_flag(sfp),			
			in7  => '0',--gbt_tx(sfp)(34),
			in8  => '0',--gbt_rx(sfp)(34),
			in9  => sfp_sda(sfp),
			in10 => sfp_scl(sfp),			
			o    => sfp_signals(sfp)
      );
	  END GENERATE;

	sfp_sel_int <= to_integer(unsigned(sfp_sel));
	sfp_out <= sfp_signals(sfp_sel_int);
					  
	debug_global_mux : global_mux
      PORT MAP (
			sel  => global_signal_sel,
			in1  => sfp_out,
			in2  => ttc_clk,
			in3  => qie_reset,		
			in4  => tx_wordclk,		
			o    => debug_out
      );
end Behavioral;











