-------------------------------------------------------------------------------
-- Title      : Constant and Function Declarations for project
--              (originally came from GBT-FPGA project)
-- Project    : 
-------------------------------------------------------------------------------
-- File       : Constant_Declaration.vhd
-- Author     : Stephen Goadhouse  <sgoadhouse@virginia.edu>
-- Company    : 
-- Created    : 2008-01-10
-- Last update: 2012-11-21
-- Platform   : 
-- Standard   : VHDL'93
-------------------------------------------------------------------------------
-- Description: 
-------------------------------------------------------------------------------
-- Copyright (c) 2012 
-------------------------------------------------------------------------------
-- Revisions  :
-- Date        Ver  Author  	  Description
-- 2008-10-01  0.1  MARIN   	  first .vhd entity definition           
-- 2009-09-01  0.2  BARON   	  added a x3 optimisation scheme
-- 2012-06-04  1.0  GOADHOUSE   Released
--------------------
-----------------------------------------------------------

--!!THIS FILE IS OVERWRITTEN IGNORE IT FOR THE MOMENT!!

LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.ALL;
use ieee.std_logic_unsigned.all;

-- Xilinx devices library:
library unisim;
use unisim.vcomponents.all;

PACKAGE ngCCM_package IS

  -- Wait States are the number of wait states in the Frameclk period, which
  -- is 40MHz/25ns. In the miniCTR2 design, the IPbus clock was 25MHz/40ns,
  -- but in this design, IPbus clock is 62.5MHz/16ns. So require less wait
  -- states.
  CONSTANT kMSB_ngCCM           : INTEGER := 11; -- For IPbus access (must match value in ADDR_MAP.v)
  CONSTANT kWAIT_ngCCM          : INTEGER := 6;  -- For IPbus access (must match value in ADDR_MAP.v)
  CONSTANT kWTICKS_ngCCM        : INTEGER := 2;  -- For IPbus access (must match value in ADDR_MAP.v)
  
  --@@@CONSTANT kNUM_WB_SLAVES  : INTEGER := 8;  -- Number of wishbone slaves

--  CONSTANT WB_SEL_BYTE         : STD_LOGIC_VECTOR(3 DOWNTO 0) := "0001";  -- Byte Transfer
--  CONSTANT WB_SEL_SHORT        : STD_LOGIC_VECTOR(3 DOWNTO 0) := "0011";  -- Short Transfer
--  CONSTANT WB_SEL_WORD         : STD_LOGIC_VECTOR(3 DOWNTO 0) := "1111";  -- Word Transfer
--
--  CONSTANT WB_TGA_MEM          : STD_LOGIC_VECTOR(1 DOWNTO 0) := "00";  -- Memory Cycle
--  CONSTANT WB_TGA_IO           : STD_LOGIC_VECTOR(1 DOWNTO 0) := "01";  -- I/O Cycle
--  CONSTANT WB_TGA_FW           : STD_LOGIC_VECTOR(1 DOWNTO 0) := "10";  -- Firmware Cycle
--  CONSTANT WB_TGA_DMA          : STD_LOGIC_VECTOR(1 DOWNTO 0) := "11";  -- DMA Cycle

  ---- Array for Wishbone data mux for data from slaves to master
  --CONSTANT kWB_DAT_BIT_SIZE     : INTEGER := 16;  -- Bit Size of WB Data Busses
  --TYPE     wb_slave_dat_typ IS ARRAY (0 TO kNUM_WB_SLAVES-1) OF STD_LOGIC_VECTOR(kWB_DAT_BIT_SIZE-1 DOWNTO 0);

  --CONSTANT kGPIO_BIT_SIZE       : INTEGER := kWB_DAT_BIT_SIZE;  -- Bit Size of GPIO

  ---- Base Address of this FPGA on the Wishbone bus.  This base address is within
  ---- the I/O space and thus only the low 16 bits are used.
  --CONSTANT kWB_BASE_ADDR        : STD_LOGIC_VECTOR(15 DOWNTO 0) := x"0000";

  ---- Base Address Mask.  AND this with wishbone address and compare to
  ---- kWB_BASE_ADDR to test for an address match.
  --CONSTANT kWB_BASE_ADDR_MASK   : STD_LOGIC_VECTOR(kWB_BASE_ADDR'range) := x"F000";

  ---- Offset Address Mask.  AND this with wishbone address and compare to
  ---- the list of offset addresses of wishbone slaves in kADDR_WBS_ARR for an address match.
  --CONSTANT kWB_OFFSET_ADDR_MASK : STD_LOGIC_VECTOR(kWB_BASE_ADDR'range) := x"0FC0";

  ---- Offset addresses for wishbone slaves
  --CONSTANT kADDR_WBS_MEM_16X16  : STD_LOGIC_VECTOR(kWB_BASE_ADDR'range) := x"0000";
  --CONSTANT kADDR_WBS_OPRT_8     : STD_LOGIC_VECTOR(kWB_BASE_ADDR'range) := x"0040";
  --CONSTANT kADDR_WBS_REGFILE    : STD_LOGIC_VECTOR(kWB_BASE_ADDR'range) := x"0080";
  --CONSTANT kADDR_WBS_GPIO1      : STD_LOGIC_VECTOR(kWB_BASE_ADDR'range) := x"0100";
  --CONSTANT kADDR_WBS_GPIO2      : STD_LOGIC_VECTOR(kWB_BASE_ADDR'range) := x"0140";
  --CONSTANT kADDR_WBS_GPIO3      : STD_LOGIC_VECTOR(kWB_BASE_ADDR'range) := x"0180";
  --CONSTANT kADDR_WBS_I2C_RBX    : STD_LOGIC_VECTOR(kWB_BASE_ADDR'range) := x"0200";
  --CONSTANT kADDR_WBS_I2C_SFP    : STD_LOGIC_VECTOR(kWB_BASE_ADDR'range) := x"0240";

  ---- Slave Array Indexes for the wishbone slaves
  --CONSTANT kWBS_MEM_16X16       : INTEGER := 0;  -- Test 32x16 memory
  --CONSTANT kWBS_OPRT_8          : INTEGER := 1;  -- Test Out Port (8 bit)
  --CONSTANT kWBS_REGFILE         : INTEGER := 2;  -- Register FILE
  --CONSTANT kWBS_GPIO1           : INTEGER := 3;  -- GPIO1 Module
  --CONSTANT kWBS_GPIO2           : INTEGER := 4;  -- GPIO2 Module
  --CONSTANT kWBS_GPIO3           : INTEGER := 5;  -- GPIO3 Module
  --CONSTANT kWBS_I2C_RBX         : INTEGER := 6;  -- I2C controller for RBX backplane
  --CONSTANT kWBS_I2C_SFP         : INTEGER := 7;  -- I2C controller for SFP port

  --TYPE addr_decode_type IS ARRAY (0 TO kNUM_WB_SLAVES-1) OF STD_LOGIC_VECTOR(kWB_BASE_ADDR'range);
  --CONSTANT kADDR_WBS_ARR        : addr_decode_type := (  -- Array of offset address comparators
  --  kADDR_WBS_MEM_16X16,
  --  kADDR_WBS_OPRT_8,
  --  kADDR_WBS_REGFILE,
  --  kADDR_WBS_GPIO1,
  --  kADDR_WBS_GPIO2,
  --  kADDR_WBS_GPIO3,
  --  kADDR_WBS_I2C_RBX,
  --  kADDR_WBS_I2C_SFP
  --  );
  
  CONSTANT kNUM_bkt_I2C_BUSSES : INTEGER := 12;

  -- Record for handling output signals
  TYPE ngccm_pin_output_typ IS RECORD
  	 -- JTAG connection
    jtag_tdi 			  : STD_LOGIC;            
    jtag_tms  			  : STD_LOGIC;
    jtag_tck           : STD_LOGIC;
    jtag_trst          : STD_LOGIC;
	 sel_addr           : STD_LOGIC_VECTOR(3 DOWNTO 0);
	 -- Redundant JTAG connection
	 sec_jtag_tdi       : STD_LOGIC;            
    sec_jtag_tms       : STD_LOGIC;
    sec_jtag_tck       : STD_LOGIC;
	 sec_jtag_trst      : STD_LOGIC;
    sec_sel_addr       : STD_LOGIC_VECTOR(3 DOWNTO 0);

	 -- Primary Fast Singnals
	 peltier  	        : STD_LOGIC_VECTOR(3 DOWNTO 0);
	 bkp_reset    	     : STD_LOGIC;
    bkp_reset_qie      : STD_LOGIC;
    bkp_wte            : STD_LOGIC;
    bkp_pwr_enable     : STD_LOGIC;
	 -- Redundant Fast Singnals
	 sec_peltier  	     : STD_LOGIC_VECTOR(3 DOWNTO 0);
	 sec_reset    	     : STD_LOGIC;
    sec_reset_qie      : STD_LOGIC;
    sec_wte            : STD_LOGIC;
    sec_pwr_enable     : STD_LOGIC;
	 
	 -- Primary I2C connection	 
    -- There is only a single SCL for a grouping of I2C "busses"	 
	  bkt_scl            : STD_LOGIC_VECTOR(2 DOWNTO 0);
    bkt_sda            : STD_LOGIC_VECTOR(kNUM_bkt_I2C_BUSSES DOWNTO 1);
	 -- Redundant I2C connection	 
    -- There is only a single SCL for a grouping of I2C "busses"	 
	  sec_scl            : STD_LOGIC;
    sec_sda            : STD_LOGIC_VECTOR(kNUM_bkt_I2C_BUSSES DOWNTO 1);
	 
	 -- Debug communication
	 test_comm	        : STD_LOGIC;
	 sec_test_comm	     : STD_LOGIC;	 		 
	 prbs_rx            : STD_LOGIC_VECTOR(19 downto 0);

  END RECORD ngccm_pin_output_typ;

  -- Not all outputs can be looped back due to lack of available bits. So
  -- remove "lb".
  TYPE ngccm_pin_input_typ IS RECORD
	 -- JTAG connection
    jtag_tdo 			  : STD_LOGIC;            
    jtag_tms  			  : STD_LOGIC;
    jtag_tck           : STD_LOGIC;
    jtag_trst          : STD_LOGIC;
	 geo_addr           : STD_LOGIC_VECTOR(3 DOWNTO 0);
	 -- Redundant JTAG connection
	 sec_jtag_tdo       : STD_LOGIC;            
    sec_jtag_tms       : STD_LOGIC;
    sec_jtag_tck       : STD_LOGIC;
	 sec_jtag_trst      : STD_LOGIC;
    sec_geo_addr       : STD_LOGIC_VECTOR(3 DOWNTO 0);

	 -- Primary Fast Singnals
	 ngccm_rev_id       : STD_LOGIC_VECTOR(2 DOWNTO 0);
	 bkp_pwr_good       : STD_LOGIC;
	 bkp_reset    	     : STD_LOGIC;
    bkp_reset_qie      : STD_LOGIC;
    bkp_wte            : STD_LOGIC;
    bkp_pwr_enable     : STD_LOGIC;
	 -- Redundant Fast Singnals
	 ngccm_neigh_rev_id : STD_LOGIC_VECTOR(2 DOWNTO 0);
	 sec_pwr_good       : STD_LOGIC;
	 sec_reset    	     : STD_LOGIC;
    sec_reset_qie      : STD_LOGIC;
    sec_wte            : STD_LOGIC;
    sec_pwr_enable     : STD_LOGIC;
	 
	 -- Primary I2C connection	 
    -- There is only a single SCL for a grouping of I2C "busses"	 
	  bkt_scl            : STD_LOGIC_VECTOR(2 DOWNTO 0);
    bkt_sda            : STD_LOGIC_VECTOR(kNUM_bkt_I2C_BUSSES DOWNTO 1);
	 -- Redundant I2C connection	 
    -- There is only a single SCL for a grouping of I2C "busses"	 
	  sec_scl            : STD_LOGIC;
    sec_sda            : STD_LOGIC_VECTOR(kNUM_bkt_I2C_BUSSES DOWNTO 1);
	 
	 -- Debug communication
	 test_comm	        : STD_LOGIC;
	 sec_test_comm	     : STD_LOGIC;	 		 
	 prbs_tx            : STD_LOGIC_VECTOR(19 downto 0);
	 
  END RECORD ngccm_pin_input_typ;
-------------------
  FUNCTION ngCCM_Pins_to_GBT_Word (CONSTANT pinss    : ngccm_pin_output_typ)  RETURN STD_LOGIC_VECTOR;
--  FUNCTION GBT_Word_to_ngCCM_Pins (CONSTANT GBT_Word : STD_LOGIC_VECTOR(83 DOWNTO 0)) RETURN ngccm_pin_input_typ;
--  FUNCTION ngCCM_Pins_to_GBT_Word (CONSTANT Ppins    : IN ngccm_pin_output_typ; CONSTANT Spins    : IN ngccm_pin_output_typ) RETURN STD_LOGIC_VECTOR;
  FUNCTION GBT_Word_to_ngCCM_Pins (CONSTANT GBT_Word : IN STD_LOGIC_VECTOR(83 DOWNTO 0)) RETURN ngccm_pin_input_typ;
 
  -- Record for passing CDCE62005 PLL signals between wb_debug.vhd and the
  -- ngCCM GBT top level.
  TYPE cdce62005_stat_typ IS RECORD
    init_done          : STD_LOGIC;
    access_done        : STD_LOGIC;
    rd_reg             : STD_LOGIC_VECTOR(3 DOWNTO 0);
    rd_dat             : STD_LOGIC_VECTOR(27 DOWNTO 0);
  END RECORD cdce62005_stat_typ;

  TYPE cdce62005_ctrl_typ IS RECORD
    init_force          : STD_LOGIC;
    reg                 : STD_LOGIC_VECTOR(3 DOWNTO 0);
    wr_dat              : STD_LOGIC_VECTOR(27 DOWNTO 0);
    access_en           : STD_LOGIC;
    access_wr_rd        : STD_LOGIC;
  END RECORD cdce62005_ctrl_typ;
  
END ngCCM_package;


PACKAGE BODY ngCCM_package IS
  -- purpose: Breakout the pins from the GBT Word. Using a function to do this
  -- so that the bit assignment is consistent among the different users.
  
  FUNCTION GBT_Word_to_ngCCM_Pins (
    CONSTANT GBT_Word : IN STD_LOGIC_VECTOR(83 DOWNTO 0))
    RETURN ngccm_pin_input_typ IS
    VARIABLE pinss : ngccm_pin_input_typ;
  BEGIN  -- FUNCTION GBT_Word_to_Pins
    
    pinss.jtag_tck  := GBT_Word(0);
    pinss.jtag_tdo  := GBT_Word(2);
    pinss.jtag_tms  := GBT_Word(4);
    pinss.jtag_trst := GBT_Word(6);

    pinss.geo_addr(0) := GBT_Word(8);
    pinss.geo_addr(1) := GBT_Word(10);
    pinss.geo_addr(2) := GBT_Word(12);
    pinss.geo_addr(3) := GBT_Word(14);

    pinss.ngccm_rev_id(0)       := GBT_Word(32);
    pinss.ngccm_rev_id(1)       := GBT_Word(34);
    pinss.ngccm_rev_id(2)       := GBT_Word(36);

    pinss.bkp_pwr_good   := GBT_Word(38);
    pinss.bkp_reset      := GBT_Word(40);
    pinss.bkp_reset_qie  := GBT_Word(42);
    pinss.bkp_wte        := GBT_Word(44);
    pinss.bkp_pwr_enable := GBT_Word(46);

    -- There is only a single SCL for a grouping of I2C "busses"
    pinss.bkt_scl(0)    := GBT_Word(72);
    pinss.bkt_sda(1) := GBT_Word(48);
    pinss.bkt_sda(2) := GBT_Word(50);
    pinss.bkt_sda(3) := GBT_Word(52);
    pinss.bkt_sda(4) := GBT_Word(54);
    pinss.bkt_sda(5) := GBT_Word(56);
    pinss.bkt_scl(1) := GBT_Word(74);
    pinss.bkt_sda(6) := GBT_Word(58);
    pinss.bkt_sda(7) := GBT_Word(60);
    pinss.bkt_sda(8) := GBT_Word(62);
    pinss.bkt_sda(9) := GBT_Word(64);
    pinss.bkt_sda(10) := GBT_Word(66);

    pinss.bkt_scl(2)    := GBT_Word(76);
    pinss.bkt_sda(11) := GBT_Word(68);
    pinss.bkt_sda(12) := GBT_Word(70);

    pinss.test_comm := GBT_Word(78);
    pinss.prbs_tx := GBT_Word(83 downto 80) & GBT_Word(31 downto 16);
    
    RETURN pinss;                  
  END FUNCTION GBT_Word_to_ngCCM_Pins;

  -- purpose: "Break-in" the pins into the GBT Word. Using a function to do this
  -- so that the bit assignment is consistent among the different users.
  
  FUNCTION ngCCM_Pins_to_GBT_Word (
    CONSTANT pinss   
	 : ngccm_pin_output_typ)
    RETURN STD_LOGIC_VECTOR IS
    VARIABLE GBT_Word : STD_LOGIC_VECTOR(83 DOWNTO 0);
  BEGIN  -- FUNCTION ngCCM_Pins_to_GBT_Word
	 -- Primary JTAG connection
    GBT_Word(0)    := pinss.jtag_tck;
    GBT_Word(2)    := pinss.jtag_tdi;
    GBT_Word(4)    := pinss.jtag_tms;
    GBT_Word(6)    := pinss.jtag_trst;
	 GBT_Word(8)    := pinss.sel_addr(0);
    GBT_Word(10)   := pinss.sel_addr(1);
    GBT_Word(12)   := pinss.sel_addr(2);
    GBT_Word(14)   := pinss.sel_addr(3);
	 -- Redundant JTAG connection
	 GBT_Word(1)    := pinss.jtag_tck;
    GBT_Word(3)    := pinss.jtag_tdi;
    GBT_Word(5)    := pinss.jtag_tms;
    GBT_Word(7)    := pinss.jtag_trst;
	 GBT_Word(9)    := pinss.sel_addr(0);
    GBT_Word(11)   := pinss.sel_addr(1);
    GBT_Word(13)   := pinss.sel_addr(2);
    GBT_Word(15)   := pinss.sel_addr(3);
	 
	 -- Primary Fast Singnals
    GBT_Word(32)   := pinss.peltier(0);
    GBT_Word(34)   := pinss.peltier(1);
    GBT_Word(36)   := pinss.peltier(2);
    GBT_Word(38)   := pinss.peltier(3);
    GBT_Word(40)   := pinss.bkp_reset;
    GBT_Word(42)   := pinss.bkp_reset_qie;
    GBT_Word(44)   := pinss.bkp_wte;
    GBT_Word(46)   := pinss.bkp_pwr_enable;	 
	 -- Redundant Fast Singnals
	 GBT_Word(33)   := pinss.peltier(0);
    GBT_Word(35)   := pinss.peltier(1);
    GBT_Word(37)   := pinss.peltier(2);
    GBT_Word(39)   := pinss.peltier(3);
    GBT_Word(41)   := pinss.bkp_reset;
    GBT_Word(43)   := pinss.sec_reset_qie;
    GBT_Word(45)   := pinss.sec_wte;
    GBT_Word(47)   := pinss.bkp_pwr_enable;

	 -- Primary I2C connection	 
    GBT_Word(48)   := pinss.bkt_sda(1);
    GBT_Word(50)   := pinss.bkt_sda(2);
    GBT_Word(52)   := pinss.bkt_sda(3);
    GBT_Word(54)   := pinss.bkt_sda(4);
    GBT_Word(56)   := pinss.bkt_sda(5);
    GBT_Word(58)   := pinss.bkt_sda(6);
    GBT_Word(60)   := pinss.bkt_sda(7);
	 GBT_Word(62)   := pinss.bkt_sda(8);
    GBT_Word(64)   := pinss.bkt_sda(9);
    GBT_Word(66)   := pinss.bkt_sda(10);
    GBT_Word(68)   := pinss.bkt_sda(11); -- internal ngCCM I2C slave
    GBT_Word(70)   := pinss.bkt_sda(12);
    GBT_Word(72)   := pinss.bkt_scl(0); 
    GBT_Word(74)   := pinss.bkt_scl(1); 
    GBT_Word(76)   := pinss.bkt_scl(2); 
	 -- Redundant I2C connection		 
	 GBT_Word(49)   := pinss.bkt_sda(1);
    GBT_Word(51)   := pinss.bkt_sda(2);
    GBT_Word(53)   := pinss.bkt_sda(3);
    GBT_Word(55)   := pinss.bkt_sda(4);
    GBT_Word(57)   := pinss.bkt_sda(5);
    GBT_Word(59)   := pinss.bkt_sda(6);
    GBT_Word(61)   := pinss.bkt_sda(7);
	 GBT_Word(63)   := pinss.bkt_sda(8);
    GBT_Word(65)   := pinss.bkt_sda(9);
    GBT_Word(67)   := pinss.bkt_sda(10);
    GBT_Word(69)   := pinss.bkt_sda(11); -- internal ngCCM I2C slave
    GBT_Word(71)   := pinss.bkt_sda(12);
    GBT_Word(73)   := pinss.bkt_scl(0); 
    GBT_Word(75)   := pinss.bkt_scl(1); 
    GBT_Word(77)   := pinss.bkt_scl(2); 
	 
	 -- Debug connection 
	 GBT_Word(31 downto 16)     := pinss.prbs_rx(15 downto 0);
    GBT_Word(78)   := pinss.test_comm; 
    GBT_Word(79)   := pinss.test_comm; 
	 GBT_Word(83 downto 80)     := pinss.prbs_rx(19 downto 16);

    RETURN GBT_Word;
  END FUNCTION ngCCM_Pins_to_GBT_Word;
--  FUNCTION ngCCM_Pins_to_GBT_Word (
--    CONSTANT Ppins : IN ngccm_pin_output_typ;
--    CONSTANT Spins : IN ngccm_pin_output_typ)
--    RETURN STD_LOGIC_VECTOR IS
--    VARIABLE GBT_Word : STD_LOGIC_VECTOR(83 DOWNTO 0);
--  BEGIN  -- FUNCTION ngCCM_Pins_to_GBT_Word
--    GBT_Word(0) := Ppins.jtag_tck;
--    GBT_Word(2) := Ppins.jtag_tdi;
--    GBT_Word(4) := Ppins.jtag_tms;
--    GBT_Word(6) := Ppins.jtag_trst;
--
--    GBT_Word(1) := Spins.jtag_tck;
--    GBT_Word(3) := Spins.jtag_tdi;
--    GBT_Word(5) := Spins.jtag_tms;
--    GBT_Word(7) := Spins.jtag_trst;
--
--    GBT_Word(8)  := Ppins.sel_addr(0);
--    GBT_Word(10) := Ppins.sel_addr(1);
--    GBT_Word(12) := Ppins.sel_addr(2);
--    GBT_Word(14) := Ppins.sel_addr(3);
--
--    GBT_Word(9)  := Spins.sel_addr(0);
--    GBT_Word(11) := Spins.sel_addr(1);
--    GBT_Word(13) := Spins.sel_addr(2);
--    GBT_Word(15) := Spins.sel_addr(3);
--
--    GBT_Word(32) := Ppins.peltier_en(1);
--    GBT_Word(34) := Ppins.peltier_en(2);
--    GBT_Word(36) := Ppins.peltier_en(3);
--    GBT_Word(38) := Ppins.peltier_en(4);
--    
--    GBT_Word(33) := Spins.peltier_en(1);
--    GBT_Word(35) := Spins.peltier_en(2);
--    GBT_Word(37) := Spins.peltier_en(3);
--    GBT_Word(39) := Spins.peltier_en(4);
--    
--    GBT_Word(40) := Ppins.reset;     
--    GBT_Word(42) := Ppins.reset_qie; 
--    GBT_Word(44) := Ppins.wte;       
--    GBT_Word(46) := Ppins.pwr_enable;
--
--    GBT_Word(41) := Spins.reset;     
--    GBT_Word(43) := Spins.reset_qie; 
--    GBT_Word(45) := Spins.wte;       
--    GBT_Word(47) := Spins.pwr_enable;
--
--    GBT_Word(72) := Ppins.blt_scl;
--    GBT_Word(48) := Ppins.blt_sda(1);
--    GBT_Word(50) := Ppins.blt_sda(2);
--    GBT_Word(52) := Ppins.blt_sda(3);
--    GBT_Word(54) := Ppins.blt_sda(4);
--    GBT_Word(56) := Ppins.blt_sda(5);
--    GBT_Word(74) := Ppins.brt_scl;
--    GBT_Word(58) := Ppins.brt_sda(1);   -- bus 6
--    GBT_Word(60) := Ppins.brt_sda(2);   -- bus 7
--    GBT_Word(62) := Ppins.brt_sda(3);   -- bus 8
--    GBT_Word(64) := Ppins.brt_sda(4);   -- bus 9
--    GBT_Word(66) := Ppins.brt_sda(5);   -- bus 10
--
--    GBT_Word(76) := Ppins.ngccm_scl;
--    GBT_Word(68) := Ppins.ngccm_sda(0); -- bus 11
--    GBT_Word(70) := Ppins.ngccm_sda(1); -- bus 12
--
--    GBT_Word(73) := Spins.blt_scl;
--    GBT_Word(49) := Spins.blt_sda(1);
--    GBT_Word(51) := Spins.blt_sda(2);
--    GBT_Word(53) := Spins.blt_sda(3);
--    GBT_Word(55) := Spins.blt_sda(4);
--    GBT_Word(57) := Spins.blt_sda(5);
--    GBT_Word(75) := Spins.brt_scl;
--    GBT_Word(59) := Spins.brt_sda(1);   -- bus 6
--    GBT_Word(61) := Spins.brt_sda(2);   -- bus 7
--    GBT_Word(63) := Spins.brt_sda(3);   -- bus 8
--    GBT_Word(65) := Spins.brt_sda(4);   -- bus 9
--    GBT_Word(67) := Spins.brt_sda(5);   -- bus 10
--
--    GBT_Word(77) := Spins.ngccm_scl;
--    GBT_Word(69) := Spins.ngccm_sda(0); -- bus 11
--    GBT_Word(71) := Spins.ngccm_sda(1); -- bus 12
--
--    GBT_Word(78) := Ppins.test_comm;
--
--    GBT_Word(79) := Spins.test_comm;
--
--    -- Fill in for the bits where the PRBS pattern will go. Set to 0 which
--    -- should be an invalid value.
--    GBT_Word(30 DOWNTO 16) := (OTHERS => '0');
--    GBT_Word(83 DOWNTO 80) := (OTHERS => '0');
--    
--    RETURN GBT_Word;
--  END FUNCTION ngCCM_Pins_to_GBT_Word;

END PACKAGE BODY ngCCM_package;
