------------------------------------------------------
-- Agnostic_Counter                                 --
--                                                  --
-- continuously increments a XX bits output vector  --
-- Synchronous reset                                --
--                                                  --
-- Author: Sophie Baron                             --
-- Date: September 25th, 2008                       --
--                                                  --
-- Modified Agnostic_21bits_Counter to have its     --
-- counter size and terminal count passed in a      --
-- GENERIC and to have an enable signal.            --
--                                                  --
-- Author: Stephen Goadhouse                        --
-- Date: Feb 3, 2012                                --
------------------------------------------------------


LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.ALL;


ENTITY Agnostic_Counter IS
  GENERIC (
    gCOUNTER_SIZE : INTEGER          := 21;               -- Bit size of the counter
    gCOUNTER_TERM : STD_LOGIC_VECTOR := '1' & x"11111");  -- Counter terminal (default to max)
  PORT(
    Clock : IN  STD_LOGIC;
    reset : IN  STD_LOGIC;
    en    : IN  STD_LOGIC := '1';                         -- default to enabled
    q     : OUT STD_LOGIC_VECTOR
    );  
END Agnostic_Counter;


ARCHITECTURE rtl OF Agnostic_Counter IS
  
  SIGNAL Counter_Temp : UNSIGNED(gCOUNTER_SIZE-1 DOWNTO 0);
  
BEGIN

  q <= STD_LOGIC_VECTOR(Counter_Temp);

  PROCESS (Clock)
  BEGIN

    IF RISING_EDGE(Clock) THEN
      IF (reset = '1') THEN
        Counter_Temp <= (OTHERS => '0');
      ELSIF (en = '1') THEN
        IF (Counter_Temp = UNSIGNED(gCOUNTER_TERM)) THEN
          Counter_Temp <= (OTHERS => '0');
        ELSE
          Counter_Temp <= Counter_Temp + 1;
        END IF;
      END IF;
    END IF;
  END PROCESS;

END rtl;
