-------------------------------------------------------------------------------
-- Title      : Sync
-- Project    : 
-------------------------------------------------------------------------------
-- File       : Sync.vhd
-- Author     : Stephen Goadhouse  <sgoadhouse@virginia.edu>
-- Company    : 
-- Created    : 2012-01-30
-- Last update: 2012-06-22
-- Platform   : 
-- Standard   : VHDL'93
-------------------------------------------------------------------------------
-- Description: Sync input signal to the clk
-------------------------------------------------------------------------------
-- Copyright (c) 2012 
-------------------------------------------------------------------------------
-- Revisions  :
-- Date        Version  Author  Description
-- 2012-01-30  1.0      SDG     Created
-------------------------------------------------------------------------------
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.ALL;

ENTITY Sync IS
  PORT(
    clk    : IN  STD_LOGIC;
    ce : IN  STD_LOGIC;
    sync_i : IN  STD_LOGIC;
    sync_o : OUT STD_LOGIC
    );
END Sync;

ARCHITECTURE rtl OF Sync IS

  SIGNAL sync_m : STD_LOGIC;

BEGIN

  -- purpose: uses two flip/flops to synchronize the input to clk
  -- shielding any potential metastability from sync_o (sync_m gets it).
  sync_proc : PROCESS (clk) IS
  BEGIN  -- PROCESS sync_proc
    IF rising_edge(clk) THEN            -- rising clock edge
			if(ce = '1')then
				sync_m <= sync_i;
				sync_o <= sync_m;
			end if;
    END IF;
  END PROCESS sync_proc;

END rtl;
