----------------------------------------------------------------------------------
-- Institute: DESY
-- Engineer: Ozgur Sahin
-- 
-- Create Date:    15:13:33 01/20/2014 
-- Design Name: 
-- Module Name:    buffer_server_com - Behavioral 
-- Project Name: 
-- Target Devices: Server <-> ngFEC
-- Tool versions: 
-- Description: The commands that will be transferred to f/e is stored here, and the communication btw server and GLIB is establish with this module
-- See Specification of ngCCM - CCMServer Communication
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created and migrated from ngFEC_module
-- Revision 0.50 - Protocol btw Server and ngFEC(described in Specification of ngCCM - CCMServer Communication) is implemented.
-- Additional Comments: 
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use work.ipbus.all;
use work.ngFEC_pack.all;
use ieee.std_logic_unsigned.all;
use ieee.numeric_std.all;
Library UNISIM;
use UNISIM.vcomponents.all;


entity buffer_server_com is
    generic( partition  :	std_logic_vector := "10101");
	port(
		ipb_clk_i		:	in 	std_logic;
		ipb_reset_i		:	in 	std_logic;
		ram_mosi			:	in		ipb_wbus;
		ram_miso			:	out	ipb_rbus;
		server_addr_o	:	out	STD_LOGIC_VECTOR (BRAM_WORD_SIZE downto 0);
		server_din_o	:	out	STD_LOGIC_VECTOR (31 downto 0);
		server_wr_o		:	out	STD_LOGIC_VECTOR (0 downto 0);
		server_dout		:	in		STD_LOGIC_VECTOR (31 downto 0);
		server_aout		:	in		STD_LOGIC_VECTOR (BRAM_WORD_SIZE downto 0);		
		input_size		:	in		STD_LOGIC_VECTOR (31 downto 0);
		output_size		:	in		STD_LOGIC_VECTOR (31 downto 0);
		control_reg		:	in		STD_LOGIC_VECTOR (31 downto 0);
		status_reg		: 	in		STD_LOGIC_VECTOR (31 downto 0);
		ngccm_state_o	:	out 	state	
	);
end buffer_server_com;

architecture Behavioral of buffer_server_com is
signal server_ack		: STD_LOGIC_VECTOR (1 downto 0)					:= (others=> '0');
begin

ipbusreadandwrite:process (ipb_clk_i,ipb_reset_i)  
	constant sr_address	: std_logic_vector (15 downto 0) := x"1194";
	constant cr_address	: std_logic_vector (15 downto 0) := x"1195"; --registers are described in Specification of ngCCM - CCMServer Communication
	constant is_address	: std_logic_vector (15 downto 0) := x"1196";
	constant os_address	: std_logic_vector (15 downto 0) := x"1197";

begin
	if(ipb_reset_i = '1') then
		ngccm_state_o  		<= refresh_s;	
		server_wr_o <= "0";
		server_ack <= "00";
	elsif rising_edge(ipb_clk_i) then
		ram_miso.ipb_err 	<='0';
		server_wr_o <= "0";
		--set the ngCCM state according to the control register 
		if control_reg  = word0 then 
			ngccm_state_o  <= refresh_s; --start 
		elsif control_reg  = word1 then
			if(status_reg = word2)then
				ngccm_state_o <= wait_s;
			else
				ngccm_state_o  <= process_s; 
			end if;
		end if;
		--write buffer: ipbus -> ram
		if ram_mosi.ipb_strobe = '1' and std_match(ram_mosi.ipb_addr(24)& ram_mosi.ipb_addr(19 downto 16), partition) then
			if ram_mosi.ipb_write = '1' then
				if (ram_mosi.ipb_addr(15 downto 0) = cr_address or ram_mosi.ipb_addr(15 downto 0) = is_address) then --drive the ack signal for control registers
					ram_miso.ipb_ack	 <= '1' ;	
				elsif control_reg = word0 and unsigned(ram_mosi.ipb_addr(BRAM_WORD_SIZE downto 0)) < 1250 then --write the commands to the buffer here
					server_wr_o 	<= "1";
					server_addr_o <= ram_mosi.ipb_addr(BRAM_WORD_SIZE downto 0);
					server_din_o 	<= ram_mosi.ipb_wdata; 
					ram_miso.ipb_ack	 <= '1' ;	
				end if;
			elsif ram_mosi.ipb_addr(15 downto 0) = cr_address then
				ram_miso.ipb_rdata <= control_reg ;	
				ram_miso.ipb_ack	 <= '1' ;	
			elsif ram_mosi.ipb_addr(15 downto 0) = is_address then
				ram_miso.ipb_rdata <= input_size ;	
				ram_miso.ipb_ack 	 <= '1' ;	
			elsif ram_mosi.ipb_addr(15 downto 0) = os_address then
				ram_miso.ipb_rdata <= output_size ;	
				ram_miso.ipb_ack 	 <= '1' ;	
			elsif ram_mosi.ipb_addr(15 downto 0) = sr_address then
				ram_miso.ipb_rdata <= status_reg ;	
				ram_miso.ipb_ack 	 <= '1' ;	
			else		
				server_addr_o <= ram_mosi.ipb_addr(BRAM_WORD_SIZE downto 0);
				case server_ack is
					when "00" => server_ack <= "01";
					when "01" => server_ack <= "10";
					when "10" => server_ack <= "11";
					when others => server_ack <= "00";
				end case;
				ram_miso.ipb_ack <= server_ack(1) and not server_ack(0) ;	
				ram_miso.ipb_rdata <= server_dout;
			end if;
		else 
			ram_miso.ipb_ack	 <= '0' ;	
			server_ack <= "00";
		end if;
	end if;
end process;

end Behavioral;

