library IEEE;
use IEEE.STD_LOGIC_1164.all;
use ieee.numeric_std.ALL;
use work.ipbus.all; 
package ngFEC_pack is
	--	sfp constants
    CONSTANT    kNUM_SFPS   : INTEGER := 12;
    CONSTANT    first_sfp   : INTEGER := 1;
    CONSTANT    last_sfp    : INTEGER := first_sfp + kNUM_SFPS - 1;
    ---------------------------------------
    
    --	partition ids
	constant no_partition	   : integer							  := 16;
	constant kNUM_I2C_ngCCM    : integer 							  := 12;
	constant SFP_PARTITION_PRI : integer							  := no_partition-4;
	constant SFP_PARTITION_SEC : integer							  := SFP_PARTITION_PRI+1;
	constant JTAG_PARTITION	   : integer							  := no_partition-2;
	constant BKP_PARTITION	   : integer							  := no_partition-1;
	constant kNUM_I2C_BUSSES   : integer							  := kNUM_I2C_ngCCM+2;
    ---------------------------------------

	constant cr_tail		      : STD_LOGIC_VECTOR(15 downto 0) := x"1195"; -- control register tail
	constant is_tail	       	: STD_LOGIC_VECTOR(15 downto 0) := x"1196"; -- input size register tail
	constant os_tail				: STD_LOGIC_VECTOR(15 downto 0) := x"1197"; -- input size register tail
	constant word0					: STD_LOGIC_VECTOR(31 downto 0) := x"00000000";
	constant word1					: STD_LOGIC_VECTOR(31 downto 0) := x"00000001";
	constant word2					: STD_LOGIC_VECTOR(31 downto 0) := x"00000002";
	constant bram_word_size		: integer                       := 11; 
  type array8X32 is array(0 to 7) of std_logic_vector(31 downto 0);
  type array12X8X32 is array(0 to 11) of array8X32;
	
	type array_32x5bit is array (31 downto 0)  of std_logic_vector(4 downto 0);
	constant part  : array_32x5bit :=  (   0 => "0"&x"0",
	                                       1 => "0"&x"1",
	                                       2 => "0"&x"2",
	                                       3 => "0"&x"3",
	                                       4 => "0"&x"4",
	                                       5 => "0"&x"5",
	                                       6 => "0"&x"6",
	                                       7 => "0"&x"7",
	                                       8 => "0"&x"8",
	                                       9 => "0"&x"9",
	                                       10 => "0"&x"a",
	                                       11 => "0"&x"b",
	                                       SFP_PARTITION_PRI => "1"&x"d",
	                                       SFP_PARTITION_SEC => "1"&x"e",
	                                       JTAG_PARTITION  => "0"&x"f",
	                                       BKP_PARTITION   => "1"&x"f",
	                                       OTHERS => "1"&x"5");
	                                                                               
	type  ttc_counter_regs  	is array (0 to 8)  of std_logic_vector(31 downto 0);
	type	state						is (process_s, wait_s, refresh_s);
	--type	part_length				is array (31 downto 0) of std_logic_vector(4 downto 0);
	type	protocol					is (CCMlike, QIE10like, GBTXlike, idle);
	type	block_state				is array (no_partition-1 downto 0) of state;
	type	ram_register			is array (no_partition-1 downto 0) of std_logic_vector(31 downto 0);--# of partitions
	type	ngccm_register			is array (no_partition-1 downto 0) of std_logic_vector(31 downto 0);--partitions
	type	ipb_in					is array (no_partition-1 downto 0) of ipb_wbus; --partitions
	type 	ipb_sfp_in				is array (kNUM_SFPS              downto 1) of ipb_in;
	type	ipb_out					is array (no_partition-1 downto 0) of ipb_rbus; --partitions
	type 	ipb_sfp_out				is array (kNUM_SFPS              downto 1) of ipb_out;
	type	I2C_sfp_ack				is array (kNUM_SFPS              downto 1) of std_logic_vector(13 downto 0); 
	type	ram_container 			is
	record
		ram_addr   					: STD_LOGIC_VECTOR (bram_word_size downto 0);
		ram_datain 					: STD_LOGIC_VECTOR (31 downto 0);
		ram_wr     					: STD_LOGIC_VECTOR (0 downto 0);
		ram_dataout					: STD_LOGIC_VECTOR (31 downto 0);
		ram_addrout					: STD_LOGIC_VECTOR (bram_word_size downto 0);
	end record;
	constant init_block_ram    : ram_container := (ram_addr     =>(others => '0'),
																  ram_datain   =>(others => '0'),
															     ram_wr       =>(others => '0'),
															     ram_dataout  =>(others => '0'),
															     ram_addrout  =>(others => '0'));
	
	type block_ram is array (no_partition-1 downto 0) of ram_container;
	constant init_ipb_rbus     : ipb_rbus       := (ipb_rdata  => (others=> '0'),
													    	      ipb_ack    => '0',
															      ipb_err    => '0');

	constant init_ipb_wbus 	   : ipb_wbus       := (ipb_addr   => (others=>'0'),
													   	      ipb_wdata  => (others=>'0'),
														         ipb_strobe => '0',
														         ipb_write  => '0');
	function ngFEC_partition_sel(signal addr : in std_logic_vector(31 downto 0)) return integer;
--	function ngFEC_partition_addr(part : integer) return std_logic_vector;
end ngFEC_pack;

package body ngFEC_pack is
	function ngFEC_partition_sel(signal addr    : in std_logic_vector(31 downto 0)) return integer is
		variable part           : integer;
	begin
	IF	std_match(addr(24), '0') then
	   IF    std_match(addr(19 DOWNTO 16), x"0") THEN part := 0;
	   ELSIF std_match(addr(19 DOWNTO 16), x"1") THEN part := 1;
	   ELSIF std_match(addr(19 DOWNTO 16), x"2") THEN part := 2; 
	   ELSIF std_match(addr(19 DOWNTO 16), x"3") THEN part := 3;
	   ELSIF std_match(addr(19 DOWNTO 16), x"4") THEN part := 4;
	   ELSIF std_match(addr(19 DOWNTO 16), x"5") THEN part := 5; 
	   ELSIF std_match(addr(19 DOWNTO 16), x"6") THEN part := 6;
	   ELSIF std_match(addr(19 DOWNTO 16), x"7") THEN part := 7;
	   ELSIF std_match(addr(19 DOWNTO 16), x"8") THEN part := 8; 
	   ELSIF std_match(addr(19 DOWNTO 16), x"9") THEN part := 9; 
	   ELSIF std_match(addr(19 DOWNTO 16), x"a") THEN part := 10;
	   ELSIF std_match(addr(19 DOWNTO 16), x"b") THEN part := 11; 
	   ELSIF std_match(addr(19 DOWNTO 16), x"f") THEN part := JTAG_PARTITION; -- reserved for JTAG
	   ELSE                                           part := 21;
	   END IF;
	ELSIF	std_match(addr(24), '1') then --redundant channels
	   IF    std_match(addr(19 DOWNTO 16), x"d") THEN part := SFP_PARTITION_PRI; -- reserved for SFP interface
	   ELSIF std_match(addr(19 DOWNTO 16), x"e") THEN part := SFP_PARTITION_SEC; -- reserved for SFP interface
	   ELSIF std_match(addr(19 DOWNTO 16), x"f") THEN part := BKP_PARTITION; -- reserved for BKP commands
	   ELSE                                           part := 21;
	   END IF;
	ELSE                                 	         part := 21;
	END IF;
  return part;
end ngFEC_partition_sel;
	
end package body;