----------------------------------------------------------------------------------
-- Company: 
-- Engineer: Ozgur Sahin
-- 
-- Create Date:    11:41:49 02/23/2015 
-- Design Name: 
-- Module Name:    ttc_counter - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: TTC command and error counter for the ngFEC FC7
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use work.ngFEC_pack.all;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
use IEEE.NUMERIC_STD.ALL;

entity ttc_counter is
    Port ( ttc_clk_i: in  STD_LOGIC;
					counters : out ttc_counter_regs;
					BC0_missing : out  STD_LOGIC;
					L1Accept : in  STD_LOGIC;
					BCntRes  : in  STD_LOGIC;
					EvCntRes : in  STD_LOGIC;
					SinErrStr: in  STD_LOGIC;
					DbErrStr : in  STD_LOGIC;
					BrcstStr : in  STD_LOGIC;
					Brcst 	  : in  STD_LOGIC_VECTOR(5 downto 0);
					reset_i  : in  STD_LOGIC);
end ttc_counter;

architecture Behavioral of ttc_counter is

	SIGNAL SinErr_counter 		: STD_LOGIC_VECTOR (31 downto 0);
	SIGNAL QIE_reset_counter 	: STD_LOGIC_VECTOR (31 downto 0);
	SIGNAL DbErr_counter 		: STD_LOGIC_VECTOR (31 downto 0);
	SIGNAL EvCnt	 				: STD_LOGIC_VECTOR (31 downto 0);
	SIGNAL BCnt 					: STD_LOGIC_VECTOR (31 downto 0);
	SIGNAL BC0_late 				: STD_LOGIC_VECTOR (31 downto 0);
	SIGNAL BC0_onTime 			: STD_LOGIC_VECTOR (31 downto 0);
	SIGNAL BC0_early 				: STD_LOGIC_VECTOR (31 downto 0);
	SIGNAL WTECnt              : STD_LOGIC_VECTOR (31 downto 0);
	SIGNAL bcnt_local              : STD_LOGIC_VECTOR (11 downto 0);
	SIGNAL miss_cnt              : STD_LOGIC_VECTOR (2 downto 0);
	signal BC0_missing_i : std_logic;
begin
--TTC counters for debugging
BC0_missing <= BC0_missing_i;
ttc_counter	:	process (	reset_i,ttc_clk_i, Brcst(0), BrcstStr ) is
	begin		
		if	reset_i = '1' then
			QIE_reset_counter	<= x"00000000";
			SinErr_counter 	<= x"00000000";
			DbErr_counter		<= x"00000000";
			BCnt					<= x"00000000";
			EvCnt					<= x"00000000";
			WTECnt				<= x"00000000";	
			BC0_missing_i <= '0';
			bcnt_local <= (others => '0');
			miss_cnt <= (others => '0');
		elsif rising_edge(ttc_clk_i) then

			--Single Err Counter 
			if SinErrStr = '1' then
				SinErr_counter <= SinErr_counter + 1;
			end if;
			--Double err counter
			if	DbErrStr = '1' then
				DbErr_counter <= DbErr_counter + 1;
			end if;
			if Brcst(0) = '1' and BrcstStr = '1' then
				QIE_reset_counter <= QIE_reset_counter + 1;
			end if;
			-- Bunch counter
			if BCntRes = '1'   then 
				BCnt <= BCnt +1;
			end if;
			if(bcnt_local = x"deb" and BCntRes = '1')then
				BC0_onTime <= BC0_onTime + 1;
			end if;
			if(bcnt_local = x"deb" and BCntRes = '0')then
				BC0_late 	<= BC0_late + 1;
			end if;
			if(bcnt_local /= x"deb" and BCntRes = '1')then
				BC0_early 	<= BC0_early+'1';
			end if;
			--event counter
			if EvCntRes = '1' then 
				EvCnt <= EvCnt +1;
			end if;
			if (brcst(4) = '1' or brcst(5) = '1') and brcstStr = '1' then
				WTECnt <= WTECnt + 1;
			end if;		
			if BCntRes = '1' then 
				BC0_missing_i <= '0';
			elsif(miss_cnt = "101")then
				BC0_missing_i <= '1';
			end if;
			if(bcnt_local = x"deb" or BCntRes = '1')then
				bcnt_local <= (others => '0');
			else
				bcnt_local <= bcnt_local + 1;
			end if;
			if(BC0_missing_i = '1')then
				miss_cnt <= (others => '0');
			elsif(bcnt_local = x"deb" and BCntRes = '0')then
				miss_cnt <= miss_cnt + 1;
			end if;
			counters(0)		<= QIE_reset_counter;
			counters(1)  	<= DbErr_counter;
			counters(2)		<= SinErr_counter;
			counters(3)		<= BCnt;
			counters(4)		<= EvCnt;
			counters(5)		<= BC0_late;
			counters(6)		<= BC0_ontime;
			counters(7)		<= BC0_early;
			counters(8)		<= WTECnt;
		end if;
	end process;
end Behavioral;

