library IEEE;
use IEEE.STD_LOGIC_1164.all;
use ieee.numeric_std.all;
use work.ipbus.all;
use work.user_package.all;

package user_addr_decode is
	function user_ipb_addr_sel(signal addr : in std_logic_vector(31 downto 0)) return integer;
end user_addr_decode;

package body user_addr_decode is
	function user_ipb_addr_sel(signal addr : in std_logic_vector(31 downto 0)) return integer is
		variable sel : integer;
	begin
		--              addr, "00------------------------------" is reserved (system ipbus fabric)
		if    std_match(addr, "010000000000000000000000--------") then  	sel := user_ipb_stat_regs;
		elsif std_match(addr, "0100000000000000000000010-------") then  	sel := user_ipb_ctrl_regs;
		elsif SFP_ENABLE(1) and std_match(addr(31 DOWNTO 0), "0110111-0000--------------------") THEN sel := 1; -- SFP port 1
        elsif SFP_ENABLE(2) and std_match(addr(31 DOWNTO 0), "0110111-0001--------------------") THEN sel := 2; -- SFP port 2
        elsif SFP_ENABLE(3) and std_match(addr(31 DOWNTO 0), "0110111-0010--------------------") THEN sel := 3; -- SFP port 3
        elsif SFP_ENABLE(4) and std_match(addr(31 DOWNTO 0), "0110111-0011--------------------") THEN sel := 4; -- SFP port 4
        elsif SFP_ENABLE(5) and std_match(addr(31 DOWNTO 0), "0110111-0100--------------------") THEN sel := 5; -- SFP port 5
        elsif SFP_ENABLE(6) and std_match(addr(31 DOWNTO 0), "0110111-0101--------------------") THEN sel := 6; -- SFP port 6
        elsif SFP_ENABLE(7) and std_match(addr(31 DOWNTO 0), "0110111-0110--------------------") THEN sel := 7; -- SFP port 7
        elsif SFP_ENABLE(8) and std_match(addr(31 DOWNTO 0), "0110111-0111--------------------") THEN sel := 8; -- SFP port 8
        elsif SFP_ENABLE(9) and std_match(addr(31 DOWNTO 0), "0110111-1000--------------------") THEN sel := 9; -- SFP port 9
        elsif SFP_ENABLE(10) and std_match(addr(31 DOWNTO 0), "0110111-1001--------------------") THEN sel := 10; -- SFP port 10
        elsif SFP_ENABLE(11) and std_match(addr(31 DOWNTO 0), "0110111-1010--------------------") THEN sel := 11; -- SFP port 11
        elsif SFP_ENABLE(12) and std_match(addr(31 DOWNTO 0), "0110111-1011--------------------") THEN sel := 12; -- SFP port 12
		else	
			sel := user_ipb_stat_regs;
		end if;
		return sel;
	end user_ipb_addr_sel;
end user_addr_decode;